/**
 * gclc-socket:net.bigeon.gclc.socket.PluggableConsoleOutput.java
 * Created on: Nov 18, 2017
 */
package net.bigeon.gclc.socket;

/*-
 * #%L
 * GCLC Socket
 * %%
 * Copyright (C) 2016 - 2018 Bigeon
 * %%
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * #L%
 */
import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayDeque;
import java.util.Deque;

import net.bigeon.gclc.manager.ConsoleOutput;

/** An output that can be hotplugged to an actual output.
 *
 * @author Emmanuel Bigeon */
public final class PluggableConsoleOutput implements ConsoleOutput {

    /** The actual output. */
    private PrintStream         out;
    /** The buffered messages. */
    private final Deque<String> messages = new ArrayDeque<>();
    /** If this output is closed. */
    private boolean             closed   = false;

    /** Default constructor. */
    public PluggableConsoleOutput() {
        //
    }

    /* (non-Javadoc)
     * @see java.lang.AutoCloseable#close() */
    @Override
    public void close() {
        closed = true;
    }

    /** Set the output to write to.
     *
     * @param output the output to set */
    public synchronized void connect(final PrintStream output) {
        out = output;
        while (!messages.isEmpty()) {
            output.print(messages.pop());
        }
    }

    /** Disconnects the output. */
    public synchronized void disconnect() {
        out = null;
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.manager.ConsoleOutput#isClosed() */
    @Override
    public boolean isClosed() {
        return closed;
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.manager.ConsoleOutput#print(java.lang.String) */
    @Override
    public synchronized void print(final String text) throws IOException {
        if (closed) {
            throw new IOException("Closed output"); //$NON-NLS-1$
        }
        if (out == null) {
            messages.add(text);
        } else {
            out.print(text);
        }
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.manager.ConsoleOutput#println() */
    @Override
    public synchronized void println() throws IOException {
        if (closed) {
            throw new IOException("Closed output"); //$NON-NLS-1$
        }
        if (out == null) {
            messages.add("\n"); //$NON-NLS-1$
        } else {
            out.println();
        }
    }

    /* (non-Javadoc)
     * @see fr.bigeon.gclc.manager.ConsoleOutput#println(java.lang.String) */
    @Override
    public synchronized void println(final String message) throws IOException {
        print(message);
        println();
    }

}
