/**
 * gclc-socket:net.bigeon.gclc.socket.SocketConsoleInterface.java
 * Created on: Nov 18, 2017
 */
package net.bigeon.gclc.socket;

/*-
 * #%L
 * GCLC Socket
 * %%
 * Copyright (C) 2016 - 2018 Bigeon
 * %%
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 * #L%
 */
import java.io.IOException;
import java.io.PrintStream;
import java.net.Socket;
import java.nio.charset.StandardCharsets;

/** The interface for socket based access to console application.
 *
 * @author Emmanuel Bigeon */
public final class SocketConsoleInterface {

    /** The application's input. */
    private final PluggableConsoleInput  input;
    /** The application's output. */
    private final PluggableConsoleOutput output;

    /** Create the interfacing object.
     *
     * @param input the input
     * @param output the output */
    public SocketConsoleInterface(final PluggableConsoleInput input,
                                  final PluggableConsoleOutput output) {
        super();
        this.input = input;
        this.output = output;
    }

    /** Connect the application's input and outputs to the socket's.
     *
     * @param socket the socket
     * @throws IOException if the connection failed */
    public void connect(final Socket socket) throws IOException {
        final PrintStream printStream = new PrintStream(socket.getOutputStream(), true,
                StandardCharsets.UTF_8.name());
        output.connect(printStream);
        input.connect(socket.getInputStream(), printStream);
    }

    /** Disconnect the input and output of the application from the socket's. */
    public void disconnect() {
        input.disconnect();
        output.disconnect();
    }
}
